# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.client_grant_allow_any_organization_enum import ClientGrantAllowAnyOrganizationEnum
from ..types.client_grant_default_for_enum import ClientGrantDefaultForEnum
from ..types.client_grant_organization_nullable_usage_enum import ClientGrantOrganizationNullableUsageEnum
from ..types.client_grant_organization_usage_enum import ClientGrantOrganizationUsageEnum
from ..types.client_grant_response_content import ClientGrantResponseContent
from ..types.client_grant_subject_type_enum import ClientGrantSubjectTypeEnum
from ..types.create_client_grant_response_content import CreateClientGrantResponseContent
from ..types.get_client_grant_response_content import GetClientGrantResponseContent
from ..types.list_client_grant_paginated_response_content import ListClientGrantPaginatedResponseContent
from ..types.update_client_grant_response_content import UpdateClientGrantResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawClientGrantsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        audience: typing.Optional[str] = None,
        client_id: typing.Optional[str] = None,
        allow_any_organization: typing.Optional[ClientGrantAllowAnyOrganizationEnum] = None,
        subject_type: typing.Optional[ClientGrantSubjectTypeEnum] = None,
        default_for: typing.Optional[ClientGrantDefaultForEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ClientGrantResponseContent, ListClientGrantPaginatedResponseContent]:
        """
        Retrieve a list of [client grants](https://auth0.com/docs/get-started/applications/application-access-to-apis-client-grants), including the scopes associated with the application/API pair.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        audience : typing.Optional[str]
            Optional filter on audience.

        client_id : typing.Optional[str]
            Optional filter on client_id.

        allow_any_organization : typing.Optional[ClientGrantAllowAnyOrganizationEnum]
            Optional filter on allow_any_organization.

        subject_type : typing.Optional[ClientGrantSubjectTypeEnum]
            The type of application access the client grant allows.

        default_for : typing.Optional[ClientGrantDefaultForEnum]
            Applies this client grant as the default for all clients in the specified group. The only accepted value is <a href="https://auth0.com/docs/get-started/applications/application-access-to-apis-client-grants#default-permissions-for-third-party-applications">`third_party_clients`</a>, which applies the grant to all third-party clients. Per-client grants for the same audience take precedence. Mutually exclusive with `client_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ClientGrantResponseContent, ListClientGrantPaginatedResponseContent]
            Client grants successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "client-grants",
            method="GET",
            params={
                "from": from_,
                "take": take,
                "audience": audience,
                "client_id": client_id,
                "allow_any_organization": allow_any_organization,
                "subject_type": subject_type,
                "default_for": default_for,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListClientGrantPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListClientGrantPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.client_grants
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    from_=_parsed_next,
                    take=take,
                    audience=audience,
                    client_id=client_id,
                    allow_any_organization=allow_any_organization,
                    subject_type=subject_type,
                    default_for=default_for,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        audience: str,
        client_id: typing.Optional[str] = OMIT,
        default_for: typing.Optional[ClientGrantDefaultForEnum] = OMIT,
        organization_usage: typing.Optional[ClientGrantOrganizationUsageEnum] = OMIT,
        allow_any_organization: typing.Optional[bool] = OMIT,
        scope: typing.Optional[typing.Sequence[str]] = OMIT,
        subject_type: typing.Optional[ClientGrantSubjectTypeEnum] = OMIT,
        authorization_details_types: typing.Optional[typing.Sequence[str]] = OMIT,
        allow_all_scopes: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateClientGrantResponseContent]:
        """
        Create a client grant for a machine-to-machine login flow. To learn more, read [Client Credential Flow](https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow).

        Parameters
        ----------
        audience : str
            The audience (API identifier) of this client grant

        client_id : typing.Optional[str]
            ID of the client.

        default_for : typing.Optional[ClientGrantDefaultForEnum]

        organization_usage : typing.Optional[ClientGrantOrganizationUsageEnum]

        allow_any_organization : typing.Optional[bool]
            If enabled, any organization can be used with this grant. If disabled (default), the grant must be explicitly assigned to the desired organizations.

        scope : typing.Optional[typing.Sequence[str]]
            Scopes allowed for this client grant.

        subject_type : typing.Optional[ClientGrantSubjectTypeEnum]

        authorization_details_types : typing.Optional[typing.Sequence[str]]
            Types of authorization_details allowed for this client grant.

        allow_all_scopes : typing.Optional[bool]
            If enabled, all scopes configured on the resource server are allowed for this grant.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateClientGrantResponseContent]
            Client grant successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "client-grants",
            method="POST",
            json={
                "client_id": client_id,
                "audience": audience,
                "default_for": default_for,
                "organization_usage": organization_usage,
                "allow_any_organization": allow_any_organization,
                "scope": scope,
                "subject_type": subject_type,
                "authorization_details_types": authorization_details_types,
                "allow_all_scopes": allow_all_scopes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateClientGrantResponseContent,
                    parse_obj_as(
                        type_=CreateClientGrantResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetClientGrantResponseContent]:
        """
        Retrieve a single [client grant](https://auth0.com/docs/get-started/applications/application-access-to-apis-client-grants), including the
        scopes associated with the application/API pair.

        Parameters
        ----------
        id : str
            The ID of the client grant to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetClientGrantResponseContent]
            Client grant successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"client-grants/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetClientGrantResponseContent,
                    parse_obj_as(
                        type_=GetClientGrantResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete the [Client Credential Flow](https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow) from your machine-to-machine application.

        Parameters
        ----------
        id : str
            ID of the client grant to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"client-grants/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        scope: typing.Optional[typing.Sequence[str]] = OMIT,
        organization_usage: typing.Optional[ClientGrantOrganizationNullableUsageEnum] = OMIT,
        allow_any_organization: typing.Optional[bool] = OMIT,
        authorization_details_types: typing.Optional[typing.Sequence[str]] = OMIT,
        allow_all_scopes: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateClientGrantResponseContent]:
        """
        Update a client grant.

        Parameters
        ----------
        id : str
            ID of the client grant to update.

        scope : typing.Optional[typing.Sequence[str]]
            Scopes allowed for this client grant.

        organization_usage : typing.Optional[ClientGrantOrganizationNullableUsageEnum]

        allow_any_organization : typing.Optional[bool]
            Controls allowing any organization to be used with this grant

        authorization_details_types : typing.Optional[typing.Sequence[str]]
            Types of authorization_details allowed for this client grant.

        allow_all_scopes : typing.Optional[bool]
            If enabled, all scopes configured on the resource server are allowed for this grant.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateClientGrantResponseContent]
            Client grant successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"client-grants/{encode_path_param(id)}",
            method="PATCH",
            json={
                "scope": scope,
                "organization_usage": organization_usage,
                "allow_any_organization": allow_any_organization,
                "authorization_details_types": authorization_details_types,
                "allow_all_scopes": allow_all_scopes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateClientGrantResponseContent,
                    parse_obj_as(
                        type_=UpdateClientGrantResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawClientGrantsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        audience: typing.Optional[str] = None,
        client_id: typing.Optional[str] = None,
        allow_any_organization: typing.Optional[ClientGrantAllowAnyOrganizationEnum] = None,
        subject_type: typing.Optional[ClientGrantSubjectTypeEnum] = None,
        default_for: typing.Optional[ClientGrantDefaultForEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ClientGrantResponseContent, ListClientGrantPaginatedResponseContent]:
        """
        Retrieve a list of [client grants](https://auth0.com/docs/get-started/applications/application-access-to-apis-client-grants), including the scopes associated with the application/API pair.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        audience : typing.Optional[str]
            Optional filter on audience.

        client_id : typing.Optional[str]
            Optional filter on client_id.

        allow_any_organization : typing.Optional[ClientGrantAllowAnyOrganizationEnum]
            Optional filter on allow_any_organization.

        subject_type : typing.Optional[ClientGrantSubjectTypeEnum]
            The type of application access the client grant allows.

        default_for : typing.Optional[ClientGrantDefaultForEnum]
            Applies this client grant as the default for all clients in the specified group. The only accepted value is <a href="https://auth0.com/docs/get-started/applications/application-access-to-apis-client-grants#default-permissions-for-third-party-applications">`third_party_clients`</a>, which applies the grant to all third-party clients. Per-client grants for the same audience take precedence. Mutually exclusive with `client_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ClientGrantResponseContent, ListClientGrantPaginatedResponseContent]
            Client grants successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "client-grants",
            method="GET",
            params={
                "from": from_,
                "take": take,
                "audience": audience,
                "client_id": client_id,
                "allow_any_organization": allow_any_organization,
                "subject_type": subject_type,
                "default_for": default_for,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListClientGrantPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListClientGrantPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.client_grants
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""

                async def _get_next():
                    return await self.list(
                        from_=_parsed_next,
                        take=take,
                        audience=audience,
                        client_id=client_id,
                        allow_any_organization=allow_any_organization,
                        subject_type=subject_type,
                        default_for=default_for,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        audience: str,
        client_id: typing.Optional[str] = OMIT,
        default_for: typing.Optional[ClientGrantDefaultForEnum] = OMIT,
        organization_usage: typing.Optional[ClientGrantOrganizationUsageEnum] = OMIT,
        allow_any_organization: typing.Optional[bool] = OMIT,
        scope: typing.Optional[typing.Sequence[str]] = OMIT,
        subject_type: typing.Optional[ClientGrantSubjectTypeEnum] = OMIT,
        authorization_details_types: typing.Optional[typing.Sequence[str]] = OMIT,
        allow_all_scopes: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateClientGrantResponseContent]:
        """
        Create a client grant for a machine-to-machine login flow. To learn more, read [Client Credential Flow](https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow).

        Parameters
        ----------
        audience : str
            The audience (API identifier) of this client grant

        client_id : typing.Optional[str]
            ID of the client.

        default_for : typing.Optional[ClientGrantDefaultForEnum]

        organization_usage : typing.Optional[ClientGrantOrganizationUsageEnum]

        allow_any_organization : typing.Optional[bool]
            If enabled, any organization can be used with this grant. If disabled (default), the grant must be explicitly assigned to the desired organizations.

        scope : typing.Optional[typing.Sequence[str]]
            Scopes allowed for this client grant.

        subject_type : typing.Optional[ClientGrantSubjectTypeEnum]

        authorization_details_types : typing.Optional[typing.Sequence[str]]
            Types of authorization_details allowed for this client grant.

        allow_all_scopes : typing.Optional[bool]
            If enabled, all scopes configured on the resource server are allowed for this grant.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateClientGrantResponseContent]
            Client grant successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "client-grants",
            method="POST",
            json={
                "client_id": client_id,
                "audience": audience,
                "default_for": default_for,
                "organization_usage": organization_usage,
                "allow_any_organization": allow_any_organization,
                "scope": scope,
                "subject_type": subject_type,
                "authorization_details_types": authorization_details_types,
                "allow_all_scopes": allow_all_scopes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateClientGrantResponseContent,
                    parse_obj_as(
                        type_=CreateClientGrantResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetClientGrantResponseContent]:
        """
        Retrieve a single [client grant](https://auth0.com/docs/get-started/applications/application-access-to-apis-client-grants), including the
        scopes associated with the application/API pair.

        Parameters
        ----------
        id : str
            The ID of the client grant to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetClientGrantResponseContent]
            Client grant successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"client-grants/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetClientGrantResponseContent,
                    parse_obj_as(
                        type_=GetClientGrantResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete the [Client Credential Flow](https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow) from your machine-to-machine application.

        Parameters
        ----------
        id : str
            ID of the client grant to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"client-grants/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        scope: typing.Optional[typing.Sequence[str]] = OMIT,
        organization_usage: typing.Optional[ClientGrantOrganizationNullableUsageEnum] = OMIT,
        allow_any_organization: typing.Optional[bool] = OMIT,
        authorization_details_types: typing.Optional[typing.Sequence[str]] = OMIT,
        allow_all_scopes: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateClientGrantResponseContent]:
        """
        Update a client grant.

        Parameters
        ----------
        id : str
            ID of the client grant to update.

        scope : typing.Optional[typing.Sequence[str]]
            Scopes allowed for this client grant.

        organization_usage : typing.Optional[ClientGrantOrganizationNullableUsageEnum]

        allow_any_organization : typing.Optional[bool]
            Controls allowing any organization to be used with this grant

        authorization_details_types : typing.Optional[typing.Sequence[str]]
            Types of authorization_details allowed for this client grant.

        allow_all_scopes : typing.Optional[bool]
            If enabled, all scopes configured on the resource server are allowed for this grant.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateClientGrantResponseContent]
            Client grant successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"client-grants/{encode_path_param(id)}",
            method="PATCH",
            json={
                "scope": scope,
                "organization_usage": organization_usage,
                "allow_any_organization": allow_any_organization,
                "authorization_details_types": authorization_details_types,
                "allow_all_scopes": allow_all_scopes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateClientGrantResponseContent,
                    parse_obj_as(
                        type_=UpdateClientGrantResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_org_group_role_assigned_object_group import (
    EventStreamCloudEventOrgGroupRoleAssignedObjectGroup,
)
from .event_stream_cloud_event_org_group_role_assigned_object_organization import (
    EventStreamCloudEventOrgGroupRoleAssignedObjectOrganization,
)
from .event_stream_cloud_event_org_group_role_assigned_object_role import (
    EventStreamCloudEventOrgGroupRoleAssignedObjectRole,
)


class EventStreamCloudEventOrgGroupRoleAssignedObject(UniversalBaseModel):
    """
    The event content.
    """

    organization: EventStreamCloudEventOrgGroupRoleAssignedObjectOrganization
    role: EventStreamCloudEventOrgGroupRoleAssignedObjectRole
    group: EventStreamCloudEventOrgGroupRoleAssignedObjectGroup
    created_at: dt.datetime = pydantic.Field()
    """
    The time at which the role was assigned to the group in the organization.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.list_user_effective_roles_response_content import ListUserEffectiveRolesResponseContent
from ...types.user_effective_role import UserEffectiveRole
from .raw_client import AsyncRawEffectiveRolesClient, RawEffectiveRolesClient

if typing.TYPE_CHECKING:
    from .sources.client import AsyncSourcesClient, SourcesClient


class EffectiveRolesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEffectiveRolesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._sources: typing.Optional[SourcesClient] = None

    @property
    def with_raw_response(self) -> RawEffectiveRolesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEffectiveRolesClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[UserEffectiveRole, ListUserEffectiveRolesResponseContent]:
        """
        Retrieve detailed list of effective roles for a user, including roles assigned directly and through group memberships.

        Parameters
        ----------
        id : str
            ID of the user to list effective roles for.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[UserEffectiveRole, ListUserEffectiveRolesResponseContent]
            User's effective roles successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.users.effective_roles.list(
            id="id",
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(id, from_=from_, take=take, request_options=request_options)

    @property
    def sources(self):
        if self._sources is None:
            from .sources.client import SourcesClient  # noqa: E402

            self._sources = SourcesClient(client_wrapper=self._client_wrapper)
        return self._sources


class AsyncEffectiveRolesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEffectiveRolesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._sources: typing.Optional[AsyncSourcesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawEffectiveRolesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEffectiveRolesClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[UserEffectiveRole, ListUserEffectiveRolesResponseContent]:
        """
        Retrieve detailed list of effective roles for a user, including roles assigned directly and through group memberships.

        Parameters
        ----------
        id : str
            ID of the user to list effective roles for.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[UserEffectiveRole, ListUserEffectiveRolesResponseContent]
            User's effective roles successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.users.effective_roles.list(
                id="id",
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(id, from_=from_, take=take, request_options=request_options)

    @property
    def sources(self):
        if self._sources is None:
            from .sources.client import AsyncSourcesClient  # noqa: E402

            self._sources = AsyncSourcesClient(client_wrapper=self._client_wrapper)
        return self._sources



# This file was auto-generated by Fern from our API Definition.

import typing

ActionTriggerTypeEnum = typing.Union[
    typing.Literal[
        "post-login",
        "credentials-exchange",
        "pre-user-registration",
        "post-user-registration",
        "post-change-password",
        "send-phone-message",
        "custom-phone-provider",
        "custom-email-provider",
        "password-reset-post-challenge",
        "custom-token-exchange",
        "event-stream",
        "password-hash-migration",
        "login-post-identifier",
        "signup-post-identifier",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.create_organization_discovery_domain_response_content import (
    CreateOrganizationDiscoveryDomainResponseContent,
)
from ...types.get_organization_discovery_domain_by_name_response_content import (
    GetOrganizationDiscoveryDomainByNameResponseContent,
)
from ...types.get_organization_discovery_domain_response_content import GetOrganizationDiscoveryDomainResponseContent
from ...types.list_organization_discovery_domains_response_content import (
    ListOrganizationDiscoveryDomainsResponseContent,
)
from ...types.organization_discovery_domain import OrganizationDiscoveryDomain
from ...types.organization_discovery_domain_status import OrganizationDiscoveryDomainStatus
from ...types.update_organization_discovery_domain_response_content import (
    UpdateOrganizationDiscoveryDomainResponseContent,
)
from .raw_client import AsyncRawDiscoveryDomainsClient, RawDiscoveryDomainsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DiscoveryDomainsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDiscoveryDomainsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDiscoveryDomainsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDiscoveryDomainsClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[OrganizationDiscoveryDomain, ListOrganizationDiscoveryDomainsResponseContent]:
        """
        Retrieve list of all organization discovery domains associated with the specified organization.
        This endpoint is subject to eventual consistency; newly created, updated, or deleted discovery domains may not immediately appear in the response.

        Parameters
        ----------
        id : str
            ID of the organization.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[OrganizationDiscoveryDomain, ListOrganizationDiscoveryDomainsResponseContent]
            Organization discovery domains retrieved successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.organizations.discovery_domains.list(
            id="id",
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(id, from_=from_, take=take, request_options=request_options)

    def create(
        self,
        id: str,
        *,
        domain: str,
        status: typing.Optional[OrganizationDiscoveryDomainStatus] = OMIT,
        use_for_organization_discovery: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationDiscoveryDomainResponseContent:
        """
        Create a new discovery domain for an organization.

        Parameters
        ----------
        id : str
            ID of the organization.

        domain : str
            The domain name to associate with the organization e.g. acme.com.

        status : typing.Optional[OrganizationDiscoveryDomainStatus]

        use_for_organization_discovery : typing.Optional[bool]
            Indicates whether this domain should be used for organization discovery.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationDiscoveryDomainResponseContent
            Organization discovery domain successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.discovery_domains.create(
            id="id",
            domain="domain",
        )
        """
        _response = self._raw_client.create(
            id,
            domain=domain,
            status=status,
            use_for_organization_discovery=use_for_organization_discovery,
            request_options=request_options,
        )
        return _response.data

    def get_by_name(
        self, id: str, discovery_domain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationDiscoveryDomainByNameResponseContent:
        """
        Retrieve details about a single organization discovery domain specified by domain name.
        This endpoint is subject to eventual consistency; newly created, updated, or deleted discovery domains may not immediately appear in the response.

        Parameters
        ----------
        id : str
            ID of the organization.

        discovery_domain : str
            Domain name of the discovery domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationDiscoveryDomainByNameResponseContent
            Organization discovery domain successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.discovery_domains.get_by_name(
            id="id",
            discovery_domain="discovery_domain",
        )
        """
        _response = self._raw_client.get_by_name(id, discovery_domain, request_options=request_options)
        return _response.data

    def get(
        self, id: str, discovery_domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationDiscoveryDomainResponseContent:
        """
        Retrieve details about a single organization discovery domain specified by ID.
        This endpoint is subject to eventual consistency; newly created, updated, or deleted discovery domains may not immediately appear in the response.

        Parameters
        ----------
        id : str
            ID of the organization.

        discovery_domain_id : str
            ID of the discovery domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationDiscoveryDomainResponseContent
            Organization discovery domain successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.discovery_domains.get(
            id="id",
            discovery_domain_id="discovery_domain_id",
        )
        """
        _response = self._raw_client.get(id, discovery_domain_id, request_options=request_options)
        return _response.data

    def delete(
        self, id: str, discovery_domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a discovery domain from an organization. This action cannot be undone.

        Parameters
        ----------
        id : str
            ID of the organization.

        discovery_domain_id : str
            ID of the discovery domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.discovery_domains.delete(
            id="id",
            discovery_domain_id="discovery_domain_id",
        )
        """
        _response = self._raw_client.delete(id, discovery_domain_id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        discovery_domain_id: str,
        *,
        status: typing.Optional[OrganizationDiscoveryDomainStatus] = OMIT,
        use_for_organization_discovery: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrganizationDiscoveryDomainResponseContent:
        """
        Update the verification status and/or use_for_organization_discovery for an organization discovery domain. The `status` field must be either `pending` or `verified`. The `use_for_organization_discovery` field can be `true` or `false` (default: `true`).

        Parameters
        ----------
        id : str
            ID of the organization.

        discovery_domain_id : str
            ID of the discovery domain to update.

        status : typing.Optional[OrganizationDiscoveryDomainStatus]

        use_for_organization_discovery : typing.Optional[bool]
            Indicates whether this domain should be used for organization discovery.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrganizationDiscoveryDomainResponseContent
            Organization discovery domain successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.discovery_domains.update(
            id="id",
            discovery_domain_id="discovery_domain_id",
        )
        """
        _response = self._raw_client.update(
            id,
            discovery_domain_id,
            status=status,
            use_for_organization_discovery=use_for_organization_discovery,
            request_options=request_options,
        )
        return _response.data


class AsyncDiscoveryDomainsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDiscoveryDomainsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDiscoveryDomainsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDiscoveryDomainsClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[OrganizationDiscoveryDomain, ListOrganizationDiscoveryDomainsResponseContent]:
        """
        Retrieve list of all organization discovery domains associated with the specified organization.
        This endpoint is subject to eventual consistency; newly created, updated, or deleted discovery domains may not immediately appear in the response.

        Parameters
        ----------
        id : str
            ID of the organization.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[OrganizationDiscoveryDomain, ListOrganizationDiscoveryDomainsResponseContent]
            Organization discovery domains retrieved successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.organizations.discovery_domains.list(
                id="id",
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(id, from_=from_, take=take, request_options=request_options)

    async def create(
        self,
        id: str,
        *,
        domain: str,
        status: typing.Optional[OrganizationDiscoveryDomainStatus] = OMIT,
        use_for_organization_discovery: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationDiscoveryDomainResponseContent:
        """
        Create a new discovery domain for an organization.

        Parameters
        ----------
        id : str
            ID of the organization.

        domain : str
            The domain name to associate with the organization e.g. acme.com.

        status : typing.Optional[OrganizationDiscoveryDomainStatus]

        use_for_organization_discovery : typing.Optional[bool]
            Indicates whether this domain should be used for organization discovery.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationDiscoveryDomainResponseContent
            Organization discovery domain successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.discovery_domains.create(
                id="id",
                domain="domain",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            id,
            domain=domain,
            status=status,
            use_for_organization_discovery=use_for_organization_discovery,
            request_options=request_options,
        )
        return _response.data

    async def get_by_name(
        self, id: str, discovery_domain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationDiscoveryDomainByNameResponseContent:
        """
        Retrieve details about a single organization discovery domain specified by domain name.
        This endpoint is subject to eventual consistency; newly created, updated, or deleted discovery domains may not immediately appear in the response.

        Parameters
        ----------
        id : str
            ID of the organization.

        discovery_domain : str
            Domain name of the discovery domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationDiscoveryDomainByNameResponseContent
            Organization discovery domain successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.discovery_domains.get_by_name(
                id="id",
                discovery_domain="discovery_domain",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_by_name(id, discovery_domain, request_options=request_options)
        return _response.data

    async def get(
        self, id: str, discovery_domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationDiscoveryDomainResponseContent:
        """
        Retrieve details about a single organization discovery domain specified by ID.
        This endpoint is subject to eventual consistency; newly created, updated, or deleted discovery domains may not immediately appear in the response.

        Parameters
        ----------
        id : str
            ID of the organization.

        discovery_domain_id : str
            ID of the discovery domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationDiscoveryDomainResponseContent
            Organization discovery domain successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.discovery_domains.get(
                id="id",
                discovery_domain_id="discovery_domain_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, discovery_domain_id, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, discovery_domain_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a discovery domain from an organization. This action cannot be undone.

        Parameters
        ----------
        id : str
            ID of the organization.

        discovery_domain_id : str
            ID of the discovery domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.discovery_domains.delete(
                id="id",
                discovery_domain_id="discovery_domain_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, discovery_domain_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        discovery_domain_id: str,
        *,
        status: typing.Optional[OrganizationDiscoveryDomainStatus] = OMIT,
        use_for_organization_discovery: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrganizationDiscoveryDomainResponseContent:
        """
        Update the verification status and/or use_for_organization_discovery for an organization discovery domain. The `status` field must be either `pending` or `verified`. The `use_for_organization_discovery` field can be `true` or `false` (default: `true`).

        Parameters
        ----------
        id : str
            ID of the organization.

        discovery_domain_id : str
            ID of the discovery domain to update.

        status : typing.Optional[OrganizationDiscoveryDomainStatus]

        use_for_organization_discovery : typing.Optional[bool]
            Indicates whether this domain should be used for organization discovery.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrganizationDiscoveryDomainResponseContent
            Organization discovery domain successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.discovery_domains.update(
                id="id",
                discovery_domain_id="discovery_domain_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            discovery_domain_id,
            status=status,
            use_for_organization_discovery=use_for_organization_discovery,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .guardian_factor_phone_factor_message_type_enum import GuardianFactorPhoneFactorMessageTypeEnum


class SetGuardianFactorPhoneMessageTypesResponseContent(UniversalBaseModel):
    message_types: typing.Optional[typing.List[GuardianFactorPhoneFactorMessageTypeEnum]] = pydantic.Field(default=None)
    """
    The list of phone factors to enable on the tenant. Can include `sms` and `voice`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_twilio_make_call import FlowActionTwilioMakeCall
from .flow_action_twilio_send_sms import FlowActionTwilioSendSms

FlowActionTwilio = typing.Union[FlowActionTwilioMakeCall, FlowActionTwilioSendSms]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import encode_path_param
from ....core.pagination import AsyncPager, SyncPager
from ....core.parse_error import ParsingError
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....core.serialization import convert_and_respect_annotation_metadata
from ....errors.bad_request_error import BadRequestError
from ....errors.forbidden_error import ForbiddenError
from ....errors.too_many_requests_error import TooManyRequestsError
from ....errors.unauthorized_error import UnauthorizedError
from ....types.action_binding import ActionBinding
from ....types.action_binding_with_ref import ActionBindingWithRef
from ....types.action_trigger_type_enum import ActionTriggerTypeEnum
from ....types.list_action_bindings_paginated_response_content import ListActionBindingsPaginatedResponseContent
from ....types.update_action_bindings_response_content import UpdateActionBindingsResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBindingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        trigger_id: ActionTriggerTypeEnum,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ActionBinding, ListActionBindingsPaginatedResponseContent]:
        """
        Retrieve the actions that are bound to a trigger. Once an action is created and deployed, it must be attached (i.e. bound) to a trigger so that it will be executed as part of a flow. The list of actions returned reflects the order in which they will be executed during the appropriate flow.

        Parameters
        ----------
        trigger_id : ActionTriggerTypeEnum
            An actions extensibility point.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned in a single request. 20 by default

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ActionBinding, ListActionBindingsPaginatedResponseContent]
            The bindings were retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            f"actions/triggers/{encode_path_param(trigger_id)}/bindings",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListActionBindingsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListActionBindingsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.bindings
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    trigger_id,
                    page=page + 1,
                    per_page=per_page,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_many(
        self,
        trigger_id: ActionTriggerTypeEnum,
        *,
        bindings: typing.Optional[typing.Sequence[ActionBindingWithRef]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateActionBindingsResponseContent]:
        """
        Update the actions that are bound (i.e. attached) to a trigger. Once an action is created and deployed, it must be attached (i.e. bound) to a trigger so that it will be executed as part of a flow. The order in which the actions are provided will determine the order in which they are executed.

        Parameters
        ----------
        trigger_id : ActionTriggerTypeEnum
            An actions extensibility point.

        bindings : typing.Optional[typing.Sequence[ActionBindingWithRef]]
            The actions that will be bound to this trigger. The order in which they are included will be the order in which they are executed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateActionBindingsResponseContent]
            The bindings were updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/triggers/{encode_path_param(trigger_id)}/bindings",
            method="PATCH",
            json={
                "bindings": convert_and_respect_annotation_metadata(
                    object_=bindings, annotation=typing.Sequence[ActionBindingWithRef], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateActionBindingsResponseContent,
                    parse_obj_as(
                        type_=UpdateActionBindingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBindingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        trigger_id: ActionTriggerTypeEnum,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ActionBinding, ListActionBindingsPaginatedResponseContent]:
        """
        Retrieve the actions that are bound to a trigger. Once an action is created and deployed, it must be attached (i.e. bound) to a trigger so that it will be executed as part of a flow. The list of actions returned reflects the order in which they will be executed during the appropriate flow.

        Parameters
        ----------
        trigger_id : ActionTriggerTypeEnum
            An actions extensibility point.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned in a single request. 20 by default

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ActionBinding, ListActionBindingsPaginatedResponseContent]
            The bindings were retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            f"actions/triggers/{encode_path_param(trigger_id)}/bindings",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListActionBindingsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListActionBindingsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.bindings
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        trigger_id,
                        page=page + 1,
                        per_page=per_page,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_many(
        self,
        trigger_id: ActionTriggerTypeEnum,
        *,
        bindings: typing.Optional[typing.Sequence[ActionBindingWithRef]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateActionBindingsResponseContent]:
        """
        Update the actions that are bound (i.e. attached) to a trigger. Once an action is created and deployed, it must be attached (i.e. bound) to a trigger so that it will be executed as part of a flow. The order in which the actions are provided will determine the order in which they are executed.

        Parameters
        ----------
        trigger_id : ActionTriggerTypeEnum
            An actions extensibility point.

        bindings : typing.Optional[typing.Sequence[ActionBindingWithRef]]
            The actions that will be bound to this trigger. The order in which they are included will be the order in which they are executed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateActionBindingsResponseContent]
            The bindings were updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/triggers/{encode_path_param(trigger_id)}/bindings",
            method="PATCH",
            json={
                "bindings": convert_and_respect_annotation_metadata(
                    object_=bindings, annotation=typing.Sequence[ActionBindingWithRef], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateActionBindingsResponseContent,
                    parse_obj_as(
                        type_=UpdateActionBindingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.create_flow_response_content import CreateFlowResponseContent
from ..types.flow_action import FlowAction
from ..types.flow_summary import FlowSummary
from ..types.get_flow_request_parameters_hydrate_enum import GetFlowRequestParametersHydrateEnum
from ..types.get_flow_response_content import GetFlowResponseContent
from ..types.list_flows_offset_paginated_response_content import ListFlowsOffsetPaginatedResponseContent
from ..types.list_flows_request_parameters_hydrate_enum import ListFlowsRequestParametersHydrateEnum
from ..types.update_flow_response_content import UpdateFlowResponseContent
from .raw_client import AsyncRawFlowsClient, RawFlowsClient

if typing.TYPE_CHECKING:
    from .executions.client import AsyncExecutionsClient, ExecutionsClient
    from .vault.client import AsyncVaultClient, VaultClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FlowsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFlowsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._executions: typing.Optional[ExecutionsClient] = None
        self._vault: typing.Optional[VaultClient] = None

    @property
    def with_raw_response(self) -> RawFlowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFlowsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        hydrate: typing.Optional[
            typing.Union[ListFlowsRequestParametersHydrateEnum, typing.Sequence[ListFlowsRequestParametersHydrateEnum]]
        ] = None,
        synchronous: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[FlowSummary, ListFlowsOffsetPaginatedResponseContent]:
        """
        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        hydrate : typing.Optional[typing.Union[ListFlowsRequestParametersHydrateEnum, typing.Sequence[ListFlowsRequestParametersHydrateEnum]]]
            hydration param

        synchronous : typing.Optional[bool]
            flag to filter by sync/async flows

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[FlowSummary, ListFlowsOffsetPaginatedResponseContent]
            Flows successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.flows.list(
            page=1,
            per_page=1,
            include_totals=True,
            hydrate=["form_count"],
            synchronous=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            hydrate=hydrate,
            synchronous=synchronous,
            request_options=request_options,
        )

    def create(
        self,
        *,
        name: str,
        actions: typing.Optional[typing.Sequence[FlowAction]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateFlowResponseContent:
        """
        Parameters
        ----------
        name : str

        actions : typing.Optional[typing.Sequence[FlowAction]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateFlowResponseContent
            Flow successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.create(
            name="name",
        )
        """
        _response = self._raw_client.create(name=name, actions=actions, request_options=request_options)
        return _response.data

    def get(
        self,
        id: str,
        *,
        hydrate: typing.Optional[
            typing.Union[GetFlowRequestParametersHydrateEnum, typing.Sequence[GetFlowRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetFlowResponseContent:
        """
        Parameters
        ----------
        id : str
            Flow identifier

        hydrate : typing.Optional[typing.Union[GetFlowRequestParametersHydrateEnum, typing.Sequence[GetFlowRequestParametersHydrateEnum]]]
            hydration param

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetFlowResponseContent
            Flow successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.get(
            id="id",
            hydrate=["form_count"],
        )
        """
        _response = self._raw_client.get(id, hydrate=hydrate, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str
            Flow id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        actions: typing.Optional[typing.Sequence[FlowAction]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateFlowResponseContent:
        """
        Parameters
        ----------
        id : str
            Flow identifier

        name : typing.Optional[str]

        actions : typing.Optional[typing.Sequence[FlowAction]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateFlowResponseContent
            Flow successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.update(
            id="id",
        )
        """
        _response = self._raw_client.update(id, name=name, actions=actions, request_options=request_options)
        return _response.data

    @property
    def executions(self):
        if self._executions is None:
            from .executions.client import ExecutionsClient  # noqa: E402

            self._executions = ExecutionsClient(client_wrapper=self._client_wrapper)
        return self._executions

    @property
    def vault(self):
        if self._vault is None:
            from .vault.client import VaultClient  # noqa: E402

            self._vault = VaultClient(client_wrapper=self._client_wrapper)
        return self._vault


class AsyncFlowsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFlowsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._executions: typing.Optional[AsyncExecutionsClient] = None
        self._vault: typing.Optional[AsyncVaultClient] = None

    @property
    def with_raw_response(self) -> AsyncRawFlowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFlowsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        hydrate: typing.Optional[
            typing.Union[ListFlowsRequestParametersHydrateEnum, typing.Sequence[ListFlowsRequestParametersHydrateEnum]]
        ] = None,
        synchronous: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[FlowSummary, ListFlowsOffsetPaginatedResponseContent]:
        """
        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        hydrate : typing.Optional[typing.Union[ListFlowsRequestParametersHydrateEnum, typing.Sequence[ListFlowsRequestParametersHydrateEnum]]]
            hydration param

        synchronous : typing.Optional[bool]
            flag to filter by sync/async flows

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[FlowSummary, ListFlowsOffsetPaginatedResponseContent]
            Flows successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.flows.list(
                page=1,
                per_page=1,
                include_totals=True,
                hydrate=["form_count"],
                synchronous=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            hydrate=hydrate,
            synchronous=synchronous,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        name: str,
        actions: typing.Optional[typing.Sequence[FlowAction]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateFlowResponseContent:
        """
        Parameters
        ----------
        name : str

        actions : typing.Optional[typing.Sequence[FlowAction]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateFlowResponseContent
            Flow successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.create(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(name=name, actions=actions, request_options=request_options)
        return _response.data

    async def get(
        self,
        id: str,
        *,
        hydrate: typing.Optional[
            typing.Union[GetFlowRequestParametersHydrateEnum, typing.Sequence[GetFlowRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetFlowResponseContent:
        """
        Parameters
        ----------
        id : str
            Flow identifier

        hydrate : typing.Optional[typing.Union[GetFlowRequestParametersHydrateEnum, typing.Sequence[GetFlowRequestParametersHydrateEnum]]]
            hydration param

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetFlowResponseContent
            Flow successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.get(
                id="id",
                hydrate=["form_count"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, hydrate=hydrate, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str
            Flow id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        actions: typing.Optional[typing.Sequence[FlowAction]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateFlowResponseContent:
        """
        Parameters
        ----------
        id : str
            Flow identifier

        name : typing.Optional[str]

        actions : typing.Optional[typing.Sequence[FlowAction]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateFlowResponseContent
            Flow successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, name=name, actions=actions, request_options=request_options)
        return _response.data

    @property
    def executions(self):
        if self._executions is None:
            from .executions.client import AsyncExecutionsClient  # noqa: E402

            self._executions = AsyncExecutionsClient(client_wrapper=self._client_wrapper)
        return self._executions

    @property
    def vault(self):
        if self._vault is None:
            from .vault.client import AsyncVaultClient  # noqa: E402

            self._vault = AsyncVaultClient(client_wrapper=self._client_wrapper)
        return self._vault



# This file was auto-generated by Fern from our API Definition.

ConnectionClientSecretOidc = str
"""
OAuth 2.0 client secret issued by the identity provider during application registration. Used to authenticate your Auth0 connection when exchanging authorization codes for tokens. May be null for public clients.
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_scope_array import ConnectionScopeArray

ConnectionFreeformScopesSalesforce = ConnectionScopeArray
"""
Additional OAuth scopes to request from Salesforce beyond the standard profile permissions. Enter valid scopes from the Salesforce documentation. Invalid scopes may cause authentication errors.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_group_created_object_0 import EventStreamCloudEventGroupCreatedObject0
from .event_stream_cloud_event_group_created_object_1 import EventStreamCloudEventGroupCreatedObject1
from .event_stream_cloud_event_group_created_object_2 import EventStreamCloudEventGroupCreatedObject2

EventStreamCloudEventGroupCreatedObject = typing.Union[
    EventStreamCloudEventGroupCreatedObject0,
    EventStreamCloudEventGroupCreatedObject1,
    EventStreamCloudEventGroupCreatedObject2,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_attribute_map_oidc import ConnectionAttributeMapOidc
from .connection_discovery_url import ConnectionDiscoveryUrl
from .connection_options_common import ConnectionOptionsCommon
from .connection_options_common_oidc import ConnectionOptionsCommonOidc
from .connection_type_enum_oidc import ConnectionTypeEnumOidc


class ConnectionOptionsOidc(ConnectionOptionsCommonOidc, ConnectionOptionsCommon):
    """
    Options for the 'oidc' connection
    """

    attribute_map: typing.Optional[ConnectionAttributeMapOidc] = None
    discovery_url: typing.Optional[ConnectionDiscoveryUrl] = None
    type: typing.Optional[ConnectionTypeEnumOidc] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_event_bridge_aws_region_enum import EventStreamEventBridgeAwsRegionEnum


class EventStreamEventBridgeConfiguration(UniversalBaseModel):
    """
    Configuration specific to an eventbridge destination.
    """

    aws_account_id: str = pydantic.Field()
    """
    AWS Account ID for EventBridge destination.
    """

    aws_region: EventStreamEventBridgeAwsRegionEnum
    aws_partner_event_source: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS Partner Event Source for EventBridge destination.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_admin_access_token_expires_in_google_apps import ConnectionAdminAccessTokenExpiresInGoogleApps
from .connection_admin_access_token_google_apps import ConnectionAdminAccessTokenGoogleApps
from .connection_admin_refresh_token_google_apps import ConnectionAdminRefreshTokenGoogleApps
from .connection_api_enable_groups_google_apps import ConnectionApiEnableGroupsGoogleApps
from .connection_api_enable_users_google_apps import ConnectionApiEnableUsersGoogleApps
from .connection_client_id_google_apps import ConnectionClientIdGoogleApps
from .connection_client_secret_google_apps import ConnectionClientSecretGoogleApps
from .connection_domain_aliases import ConnectionDomainAliases
from .connection_domain_google_apps import ConnectionDomainGoogleApps
from .connection_ext_agreed_terms_google_apps import ConnectionExtAgreedTermsGoogleApps
from .connection_ext_groups_google_apps import ConnectionExtGroupsGoogleApps
from .connection_ext_is_admin_google_apps import ConnectionExtIsAdminGoogleApps
from .connection_ext_is_suspended_google_apps import ConnectionExtIsSuspendedGoogleApps
from .connection_handle_login_from_social_google_apps import ConnectionHandleLoginFromSocialGoogleApps
from .connection_icon_url_google_apps import ConnectionIconUrlGoogleApps
from .connection_options_common import ConnectionOptionsCommon
from .connection_scope_google_apps import ConnectionScopeGoogleApps
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_tenant_domain_google_apps import ConnectionTenantDomainGoogleApps
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsGoogleApps(ConnectionOptionsCommon):
    """
    Options for the 'google-apps' connection
    """

    admin_access_token: typing.Optional[ConnectionAdminAccessTokenGoogleApps] = None
    admin_access_token_expiresin: typing.Optional[ConnectionAdminAccessTokenExpiresInGoogleApps] = None
    admin_refresh_token: typing.Optional[ConnectionAdminRefreshTokenGoogleApps] = None
    allow_setting_login_scopes: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true, allows customization of OAuth scopes requested during user login. Custom scopes are appended to the mandatory email and profile scopes. When false or omitted, only the default email and profile scopes are used. This property is automatically enabled when Token Vault or Connected Accounts features are activated.
    """

    api_enable_groups: typing.Optional[ConnectionApiEnableGroupsGoogleApps] = None
    api_enable_users: typing.Optional[ConnectionApiEnableUsersGoogleApps] = None
    client_id: ConnectionClientIdGoogleApps
    client_secret: typing.Optional[ConnectionClientSecretGoogleApps] = None
    domain: typing.Optional[ConnectionDomainGoogleApps] = None
    domain_aliases: typing.Optional[ConnectionDomainAliases] = None
    email: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the OAuth flow requests the `email` scope.
    """

    ext_agreed_terms: typing.Optional[ConnectionExtAgreedTermsGoogleApps] = None
    ext_groups: typing.Optional[ConnectionExtGroupsGoogleApps] = None
    ext_groups_extended: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Controls whether enriched group entries include `id`, `email`, `name` (true) or only the group name (false); can only be set when `ext_groups` is true.
    """

    ext_is_admin: typing.Optional[ConnectionExtIsAdminGoogleApps] = None
    ext_is_suspended: typing.Optional[ConnectionExtIsSuspendedGoogleApps] = None
    handle_login_from_social: typing.Optional[ConnectionHandleLoginFromSocialGoogleApps] = None
    icon_url: typing.Optional[ConnectionIconUrlGoogleApps] = None
    map_user_id_to_id: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Determines how Auth0 generates the user_id for Google Workspace users. When false (default), the user's email address is used. When true, Google's stable numeric user ID is used instead, which persists even if the user's email changes. This setting can only be configured when creating the connection and cannot be changed afterward.
    """

    profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the OAuth flow requests the `profile` scope.
    """

    scope: typing.Optional[ConnectionScopeGoogleApps] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    tenant_domain: typing.Optional[ConnectionTenantDomainGoogleApps] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_organizations_roles_members_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "organizations.roles.members.list_.0"
    client = get_client(test_id)
    client.organizations.roles.members.list(
        id="id",
        role_id="role_id",
        from_="from",
        take=1,
        fields="fields",
        include_fields=True,
    )
    verify_request_count(
        test_id,
        "GET",
        "/organizations/id/roles/role_id/members",
        {"from": "from", "take": "1", "fields": "fields", "include_fields": "true"},
        1,
    )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .event_stream_response_content import EventStreamResponseContent


class ListEventStreamsResponseContent(UniversalBaseModel):
    event_streams: typing_extensions.Annotated[
        typing.Optional[typing.List[EventStreamResponseContent]],
        FieldMetadata(alias="eventStreams"),
        pydantic.Field(alias="eventStreams"),
    ] = None
    next: typing.Optional[str] = pydantic.Field(default=None)
    """
    Opaque identifier for use with the <i>from</i> query parameter for the next page of results.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_error_code_enum import EventStreamCloudEventErrorCodeEnum


class EventStreamCloudEventErrorDetail(UniversalBaseModel):
    """
    Error details.
    """

    code: EventStreamCloudEventErrorCodeEnum
    message: str = pydantic.Field()
    """
    Human-readable error message.
    """

    offset: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cursor at the time of the error (when available). Can be used to resume from this position.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

ConnectionUserinfoEndpoint = str
"""
Optional URL of the identity provider's UserInfo endpoint. When configured with attribute mapping, Auth0 calls this endpoint to retrieve additional user profile claims using the access token.
"""



from .conftest import get_client, verify_request_count


def test_flows_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "flows.list_.0"
    client = get_client(test_id)
    client.flows.list(
        page=1,
        per_page=1,
        include_totals=True,
        hydrate=["form_count"],
        synchronous=True,
    )
    verify_request_count(
        test_id,
        "GET",
        "/flows",
        {"page": "1", "per_page": "1", "include_totals": "true", "hydrate": "form_count", "synchronous": "true"},
        1,
    )


def test_flows_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "flows.create.0"
    client = get_client(test_id)
    client.flows.create(
        name="name",
    )
    verify_request_count(test_id, "POST", "/flows", None, 1)


def test_flows_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "flows.get.0"
    client = get_client(test_id)
    client.flows.get(
        id="id",
        hydrate=["form_count"],
    )
    verify_request_count(test_id, "GET", "/flows/id", {"hydrate": "form_count"}, 1)


def test_flows_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "flows.delete.0"
    client = get_client(test_id)
    client.flows.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/flows/id", None, 1)


def test_flows_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "flows.update.0"
    client = get_client(test_id)
    client.flows.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/flows/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .get_default_canonical_domain_response_content import GetDefaultCanonicalDomainResponseContent
from .get_default_custom_domain_response_content import GetDefaultCustomDomainResponseContent

GetDefaultDomainResponseContent = typing.Union[
    GetDefaultCustomDomainResponseContent, GetDefaultCanonicalDomainResponseContent
]



# This file was auto-generated by Fern from our API Definition.

import logging
import typing

LogLevel = typing.Literal["debug", "info", "warn", "error"]

_LOG_LEVEL_MAP: typing.Dict[LogLevel, int] = {
    "debug": 1,
    "info": 2,
    "warn": 3,
    "error": 4,
}


class ILogger(typing.Protocol):
    def debug(self, message: str, **kwargs: typing.Any) -> None: ...
    def info(self, message: str, **kwargs: typing.Any) -> None: ...
    def warn(self, message: str, **kwargs: typing.Any) -> None: ...
    def error(self, message: str, **kwargs: typing.Any) -> None: ...


class ConsoleLogger:
    _logger: logging.Logger

    def __init__(self) -> None:
        self._logger = logging.getLogger("fern")
        if not self._logger.handlers:
            handler = logging.StreamHandler()
            handler.setFormatter(logging.Formatter("%(levelname)s - %(message)s"))
            self._logger.addHandler(handler)
            self._logger.setLevel(logging.DEBUG)

    def debug(self, message: str, **kwargs: typing.Any) -> None:
        self._logger.debug(message, extra=kwargs)

    def info(self, message: str, **kwargs: typing.Any) -> None:
        self._logger.info(message, extra=kwargs)

    def warn(self, message: str, **kwargs: typing.Any) -> None:
        self._logger.warning(message, extra=kwargs)

    def error(self, message: str, **kwargs: typing.Any) -> None:
        self._logger.error(message, extra=kwargs)


class LogConfig(typing.TypedDict, total=False):
    level: LogLevel
    logger: ILogger
    silent: bool


class Logger:
    _level: int
    _logger: ILogger
    _silent: bool

    def __init__(self, *, level: LogLevel, logger: ILogger, silent: bool) -> None:
        self._level = _LOG_LEVEL_MAP[level]
        self._logger = logger
        self._silent = silent

    def _should_log(self, level: LogLevel) -> bool:
        return not self._silent and self._level <= _LOG_LEVEL_MAP[level]

    def is_debug(self) -> bool:
        return self._should_log("debug")

    def is_info(self) -> bool:
        return self._should_log("info")

    def is_warn(self) -> bool:
        return self._should_log("warn")

    def is_error(self) -> bool:
        return self._should_log("error")

    def debug(self, message: str, **kwargs: typing.Any) -> None:
        if self.is_debug():
            self._logger.debug(message, **kwargs)

    def info(self, message: str, **kwargs: typing.Any) -> None:
        if self.is_info():
            self._logger.info(message, **kwargs)

    def warn(self, message: str, **kwargs: typing.Any) -> None:
        if self.is_warn():
            self._logger.warn(message, **kwargs)

    def error(self, message: str, **kwargs: typing.Any) -> None:
        if self.is_error():
            self._logger.error(message, **kwargs)


_default_logger: Logger = Logger(level="info", logger=ConsoleLogger(), silent=True)


def create_logger(config: typing.Optional[typing.Union[LogConfig, Logger]] = None) -> Logger:
    if config is None:
        return _default_logger
    if isinstance(config, Logger):
        return config
    return Logger(
        level=config.get("level", "info"),
        logger=config.get("logger", ConsoleLogger()),
        silent=config.get("silent", True),
    )



from .conftest import get_client, verify_request_count


def test_users_authenticators_delete_all() -> None:
    """Test deleteAll endpoint with WireMock"""
    test_id = "users.authenticators.delete_all.0"
    client = get_client(test_id)
    client.users.authenticators.delete_all(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/users/id/authenticators", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .log_date import LogDate
from .log_details import LogDetails
from .log_location_info import LogLocationInfo
from .log_security_context import LogSecurityContext


class GetLogResponseContent(UniversalBaseModel):
    date: typing.Optional[LogDate] = None
    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of event.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of this event.
    """

    connection: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the connection the event relates to.
    """

    connection_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the connection the event relates to.
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the client (application).
    """

    client_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the client (application).
    """

    ip: typing.Optional[str] = pydantic.Field(default=None)
    """
    IP address of the log event source.
    """

    hostname: typing.Optional[str] = pydantic.Field(default=None)
    """
    Hostname the event applies to.
    """

    user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the user involved in the event.
    """

    user_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the user involved in the event.
    """

    audience: typing.Optional[str] = pydantic.Field(default=None)
    """
    API audience the event applies to.
    """

    scope: typing.Optional[str] = pydantic.Field(default=None)
    """
    Scope permissions applied to the event.
    """

    strategy: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the strategy involved in the event.
    """

    strategy_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of strategy involved in the event.
    """

    log_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique ID of the event.
    """

    is_mobile: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isMobile"),
        pydantic.Field(
            alias="isMobile",
            description="Whether the client was a mobile device (true) or desktop/laptop/server (false).",
        ),
    ] = None
    """
    Whether the client was a mobile device (true) or desktop/laptop/server (false).
    """

    details: typing.Optional[LogDetails] = None
    user_agent: typing.Optional[str] = pydantic.Field(default=None)
    """
    User agent string from the client device that caused the event.
    """

    security_context: typing.Optional[LogSecurityContext] = None
    location_info: typing.Optional[LogLocationInfo] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.bot_detection_allowlist import BotDetectionAllowlist
from ...types.bot_detection_challenge_policy_password_flow_enum import BotDetectionChallengePolicyPasswordFlowEnum
from ...types.bot_detection_challenge_policy_password_reset_flow_enum import (
    BotDetectionChallengePolicyPasswordResetFlowEnum,
)
from ...types.bot_detection_challenge_policy_passwordless_flow_enum import (
    BotDetectionChallengePolicyPasswordlessFlowEnum,
)
from ...types.bot_detection_level_enum import BotDetectionLevelEnum
from ...types.bot_detection_monitoring_mode_enabled import BotDetectionMonitoringModeEnabled
from ...types.get_bot_detection_settings_response_content import GetBotDetectionSettingsResponseContent
from ...types.update_bot_detection_settings_response_content import UpdateBotDetectionSettingsResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBotDetectionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBotDetectionSettingsResponseContent]:
        """
        Get the Bot Detection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBotDetectionSettingsResponseContent]
            Bot detection configuration retrieved successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            "attack-protection/bot-detection",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBotDetectionSettingsResponseContent,
                    parse_obj_as(
                        type_=GetBotDetectionSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        bot_detection_level: typing.Optional[BotDetectionLevelEnum] = OMIT,
        challenge_password_policy: typing.Optional[BotDetectionChallengePolicyPasswordFlowEnum] = OMIT,
        challenge_passwordless_policy: typing.Optional[BotDetectionChallengePolicyPasswordlessFlowEnum] = OMIT,
        challenge_password_reset_policy: typing.Optional[BotDetectionChallengePolicyPasswordResetFlowEnum] = OMIT,
        allowlist: typing.Optional[BotDetectionAllowlist] = OMIT,
        monitoring_mode_enabled: typing.Optional[BotDetectionMonitoringModeEnabled] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateBotDetectionSettingsResponseContent]:
        """
        Update the Bot Detection configuration of your tenant.

        Parameters
        ----------
        bot_detection_level : typing.Optional[BotDetectionLevelEnum]

        challenge_password_policy : typing.Optional[BotDetectionChallengePolicyPasswordFlowEnum]

        challenge_passwordless_policy : typing.Optional[BotDetectionChallengePolicyPasswordlessFlowEnum]

        challenge_password_reset_policy : typing.Optional[BotDetectionChallengePolicyPasswordResetFlowEnum]

        allowlist : typing.Optional[BotDetectionAllowlist]

        monitoring_mode_enabled : typing.Optional[BotDetectionMonitoringModeEnabled]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateBotDetectionSettingsResponseContent]
            Bot detection configuration successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "attack-protection/bot-detection",
            method="PATCH",
            json={
                "bot_detection_level": bot_detection_level,
                "challenge_password_policy": challenge_password_policy,
                "challenge_passwordless_policy": challenge_passwordless_policy,
                "challenge_password_reset_policy": challenge_password_reset_policy,
                "allowlist": allowlist,
                "monitoring_mode_enabled": monitoring_mode_enabled,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBotDetectionSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateBotDetectionSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBotDetectionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBotDetectionSettingsResponseContent]:
        """
        Get the Bot Detection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBotDetectionSettingsResponseContent]
            Bot detection configuration retrieved successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attack-protection/bot-detection",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBotDetectionSettingsResponseContent,
                    parse_obj_as(
                        type_=GetBotDetectionSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        bot_detection_level: typing.Optional[BotDetectionLevelEnum] = OMIT,
        challenge_password_policy: typing.Optional[BotDetectionChallengePolicyPasswordFlowEnum] = OMIT,
        challenge_passwordless_policy: typing.Optional[BotDetectionChallengePolicyPasswordlessFlowEnum] = OMIT,
        challenge_password_reset_policy: typing.Optional[BotDetectionChallengePolicyPasswordResetFlowEnum] = OMIT,
        allowlist: typing.Optional[BotDetectionAllowlist] = OMIT,
        monitoring_mode_enabled: typing.Optional[BotDetectionMonitoringModeEnabled] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateBotDetectionSettingsResponseContent]:
        """
        Update the Bot Detection configuration of your tenant.

        Parameters
        ----------
        bot_detection_level : typing.Optional[BotDetectionLevelEnum]

        challenge_password_policy : typing.Optional[BotDetectionChallengePolicyPasswordFlowEnum]

        challenge_passwordless_policy : typing.Optional[BotDetectionChallengePolicyPasswordlessFlowEnum]

        challenge_password_reset_policy : typing.Optional[BotDetectionChallengePolicyPasswordResetFlowEnum]

        allowlist : typing.Optional[BotDetectionAllowlist]

        monitoring_mode_enabled : typing.Optional[BotDetectionMonitoringModeEnabled]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateBotDetectionSettingsResponseContent]
            Bot detection configuration successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attack-protection/bot-detection",
            method="PATCH",
            json={
                "bot_detection_level": bot_detection_level,
                "challenge_password_policy": challenge_password_policy,
                "challenge_passwordless_policy": challenge_passwordless_policy,
                "challenge_password_reset_policy": challenge_password_reset_policy,
                "allowlist": allowlist,
                "monitoring_mode_enabled": monitoring_mode_enabled,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBotDetectionSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateBotDetectionSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionThumbprints = typing.List[str]
"""
Array of certificate thumbprints (SHA-128/SHA-256/SHA-512 hex hashes) for validating SAML signatures. Used with WS-Federation protocol. Maximum 20 thumbprints. Each thumbprint must be a hexadecimal string.
"""
